package com.example.android.backup;

import android.app.backup.BackupAgent;
import android.app.backup.BackupDataInput;
import android.app.backup.BackupDataOutput;
import android.os.ParcelFileDescriptor;
import android.util.Log;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;

import static com.example.android.backup.FileBackupAgent.FILENAME;
import static com.example.android.backup.FileBackupAgent.LOCK;

public class DataBackupAgent extends BackupAgent {
	private static final int AGENT_VERSION = 1;
	private static final String APP_DATA_KEY = "alldata";
	private int mCount;
	private File mDataFile;
	private static final String TAG = "MyBackupAgent";

	@Override
	public void onCreate() {
		mDataFile = new File(getFilesDir(), FILENAME);
	}

	@Override
	public void onBackup(ParcelFileDescriptor oldState, BackupDataOutput data,
			ParcelFileDescriptor newState) throws IOException {
		Log.d(TAG, "Backup...");
		synchronized (LOCK) {
			RandomAccessFile file = new RandomAccessFile(mDataFile, "r");
			mCount = file.readInt();
			file.close();
		}

		boolean doBackup = (oldState == null);
		if (!doBackup) {
			doBackup = compareStateFile(oldState);
		}

		if (doBackup) {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();

			DataOutputStream dos = new DataOutputStream(baos);
			dos.writeInt(mCount);

			byte[] buffer = baos.toByteArray();
			int len = buffer.length;
			data.writeEntityHeader(APP_DATA_KEY, len);
			data.writeEntityData(buffer, len);
		}

		writeStateFile(newState);
	}

	boolean compareStateFile(ParcelFileDescriptor oldState) {
		FileInputStream fis = new FileInputStream(oldState.getFileDescriptor());
		DataInputStream dis = new DataInputStream(fis);

		try {
			int stateVersion = dis.readInt();
			if (stateVersion > AGENT_VERSION) {
				return true;
			}

			int lastCount = dis.readInt();

			return (lastCount != mCount);
		} catch (IOException e) {
			return true;
		} finally {
			try {
				dis.close();
			} catch (IOException e) {
			}
		}
	}

	void writeStateFile(ParcelFileDescriptor stateFile) throws IOException {
		FileOutputStream fos = new FileOutputStream(
				stateFile.getFileDescriptor());
		DataOutputStream dos = new DataOutputStream(fos);

		dos.writeInt(AGENT_VERSION);
		dos.writeInt(mCount);
		dos.close();
	}

	@Override
	public void onRestore(BackupDataInput data, int appVersionCode,
			ParcelFileDescriptor newState) throws IOException {
		Log.d(TAG, "Restore...");
		while (data.readNextHeader()) {
			String key = data.getKey();
			int dataSize = data.getDataSize();

			if (APP_DATA_KEY.equals(key)) {
				byte[] dataBuf = new byte[dataSize];
				data.readEntityData(dataBuf, 0, dataSize);
				ByteArrayInputStream bais = new ByteArrayInputStream(dataBuf);
				DataInputStream din = new DataInputStream(bais);

				mCount = din.readInt();
				MainActivity.sendBroadcast(this, mCount);

				synchronized (LOCK) {
					RandomAccessFile file = new RandomAccessFile(mDataFile,
							"rw");
					file.setLength(0L);
					file.writeInt(mCount);
					file.close();
				}
			} else {
				data.skipEntityData();
			}
		}

		writeStateFile(newState);
	}
}
