package com.example.android.backup;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import android.os.Bundle;
import android.os.Handler;
import android.app.Activity;
import android.app.backup.BackupManager;
import android.app.backup.RestoreObserver;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.widget.TextView;

import static com.example.android.backup.FileBackupAgent.FILENAME;
import static com.example.android.backup.FileBackupAgent.LOCK;

public class MainActivity extends Activity {
	private static final String ACTION_REFLESH = "com.example.android.backup.ACTION_REFLESH";
	private static final String KEY_COUNT = "count";
	private static final String TAG = "MainActivity";
	private final Handler mHandler = new Handler();
	private final BroadcastReceiver mReceiver = new BroadcastReceiver() {
		@Override
		public void onReceive(Context context, Intent intent) {
			int workCount;
			if (intent.hasExtra(KEY_COUNT)) {
				workCount = intent.getIntExtra(KEY_COUNT, 0);
			} else {
				workCount = readCount();
			}
			final int count = workCount;
			mHandler.post(new Runnable() {
				@Override
				public void run() {
					TextView view = (TextView) findViewById(R.id.textView);
					view.setText("Current count: " + count);
				}
			});
		}
	};
	private final IntentFilter mFilter = new IntentFilter(ACTION_REFLESH);

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
	}

	@Override
	protected void onResume() {
		super.onResume();
		registerReceiver(mReceiver, mFilter);
		int count = readCount();
		sendBroadcast(this, count);
	}

	@Override
	protected void onPause() {
		super.onPause();
		unregisterReceiver(mReceiver);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	public void onClick(View view) {
		if (view.getId() == R.id.increment) {
			increment();
		} else if (view.getId() == R.id.backup) {
			BackupManager.dataChanged(getPackageName());
		} else if (view.getId() == R.id.restore) {
			new BackupManager(this).requestRestore(new RestoreObserver() {
				@Override
				public void onUpdate(int nowBeingRestored, String currentPackage) {
					Log.d(TAG, "onUpdate: nowBeingRestored=" + nowBeingRestored + " currentPackage=" + currentPackage);
				}

				@Override
				public void restoreFinished(int error) {
					Log.d(TAG, "onrestoreFinished: error=" + error);
				}

				@Override
				public void restoreStarting(int numPackages) {
					Log.d(TAG, "restoreStarting: numPackages=" + numPackages);
				}
			});
		}
	}

	private int readCount() {
		int count = 0;
		try {
			synchronized (LOCK) {
				File file = new File(getFilesDir(), FILENAME);
				if (file.exists()) {
					FileInputStream fis = openFileInput(FILENAME);
					DataInputStream dis = new DataInputStream(fis);
					count = dis.readInt();
					dis.close();
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		return count;
	}

	private void increment() {
		int count = 0;
		try {
			synchronized (LOCK) {
				count = readCount();
				count++;
				FileOutputStream fos = openFileOutput(FILENAME,
						Context.MODE_PRIVATE);
				DataOutputStream dos = new DataOutputStream(fos);
				dos.writeInt(count);
				dos.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		sendBroadcast(this, count);
	}

	static void sendBroadcast(Context context, int count) {
		Intent intent = new Intent(ACTION_REFLESH);
		intent.putExtra(KEY_COUNT, count);
		context.sendBroadcast(intent);
	}

	static void sendBroadcast(Context context) {
		Intent intent = new Intent(ACTION_REFLESH);
		context.sendBroadcast(intent);
	}
}
